import logging
from feeluown.player_mode import PlayerModeBase

from .model import NUserModel


logger = logging.getLogger(__name__)


class FM_mode(PlayerModeBase):
    def __init__(self, app):
        super().__init__(app)
        self.player = app.player
        self._name = 'FM'
        self._songs = []

    @property
    def name(self):
        return self._name

    def on_playlist_finished(self):
        logger.debug('fm mode: playlist finished')
        song = self._get_song()
        self.player.other_mode_play(song)

    def load(self):
        self.player.stop()
        song = self._get_song()
        if song is not None:
            self.player.other_mode_play(song)
        else:
            self.unload()

    def _get_song(self):
        if not self._songs:
            self._songs = NUserModel.get_fm_song()
        song = self._songs.pop(0)
        return song



import datetime
import json
import logging
import os

from feeluown.model import SongModel, PlaylistModel

from .api import api
from .consts import USERS_INFO_FILE

logger = logging.getLogger(__name__)


class NSongModel(SongModel):
    _api = api

    def __init__(self, mid, title, length, artists_model, album_model,
                 mvid=0, url=None):
        self._mid = mid
        self._title = title
        self._url = None
        self._candidate_url = url
        self._length = length
        self.artists = artists_model
        self.album = album_model
        self.mvid = mid

        self._start_time = datetime.datetime.now()

    @property
    def mid(self):
        return self._mid

    @property
    def title(self):
        return self._title

    @property
    def artists_name(self):
        name = []
        for artist in self.artists:
            name.append(artist.name)
        return ', '.join(name)

    @property
    def album_name(self):
        return self.album.name

    @property
    def album_img(self):
        if not self.album._img:
            logger.debug('songs has no album img, so get detail')
            self.get_detail()
        return self.album.img

    @property
    def length(self):
        return self._length

    @property
    def url(self):
        if self._url is not None:
            now = datetime.datetime.now()
            if (now - self._start_time).total_seconds() / 60 > 10:
                logger.warning('%s url maybe outdated.' % (self.title))
                self._url = None
                return self.url
            return self._url
        data = self._api.weapi_songs_url([self.mid])
        if data is not None:
            if data['code'] == 200:
                # get when needed as url will be invalid several minute later
                url = data['data'][0]['url']
                if url is None:
                    return self.candidate_url

                self._start_time = datetime.datetime.now()
                self._url = url
                return url
            if data['code'] == 404:
                return self.candidate_url
        return self._url

    @property
    def candidate_url(self):
        if not self._candidate_url:
            self.get_detail()
        return self._candidate_url

    def get_detail(self):
        data = self._api.song_detail(self.mid)
        if data is not None:
            song = data['songs'][0]
            self._candidate_url = song['mp3Url']
            self.album._img = song['album']['picUrl']

    @classmethod
    def mv_available(cls, mvid):
        if mvid != 0:
            return True
        return False

    @classmethod
    def get(cls, mid):
        data = cls._api.song_detail(mid)
        return cls.create(data)

    @classmethod
    def create(cls, data):
        if data is None or not len(data['songs']):
            return None
        song_data = data['songs'][0]
        return cls.pure_create(song_data)

    @classmethod
    def pure_create(cls, song_data):
        mid = song_data['id']
        title = song_data['name']
        url = song_data.get('mp3Url', None)
        length = song_data['duration']
        album = NAlbumModel.create_from_brief(song_data['album'])
        artists = [NArtistModel(x['id'], x['name'])
                   for x in song_data['artists']]
        mvid = song_data['mvid']
        model = cls(mid, title, length, artists, album, mvid, url)
        return model

    @classmethod
    def batch_create(cls, datas):
        return [cls.pure_create(data) for data in datas]

    @classmethod
    def search(cls, text):
        data = cls._api.search(text)
        songs = []
        if data is not None:
            if data['result']['songCount']:
                songs = data['result']['songs']
        return cls.batch_create(songs)


class NAlbumModel(object):
    _api = api

    def __init__(self, bid, name, artists_name, songs=[], img='', desc=''):
        super().__init__()

        self.bid = bid
        self._name = name
        self._artists_name = artists_name
        self._songs = songs
        self._img = img
        self._desc = desc

    @property
    def name(self):
        return self._name

    @property
    def artists_name(self):
        return self._artists_name

    @property
    def img(self):
        if not self._img:
            logger.debug('album has no img, so get detail')
            self.get_detail()
        return self._img

    @property
    def songs(self):
        if not self._songs:
            logger.debug('album has no songs, so get detail')
            self.get_detail()
        return self._songs

    @property
    def desc(self):
        if not self._desc:
            logger.debug('album has no desc, so get detail')
            self.get_detail()
        return self._desc

    @classmethod
    def create_from_brief(cls, data):
        pid = data['id']
        name = data['name']
        if 'artists' in data:
            artists_name = ', '.join([x['name'] for x in data['artists']])
        else:
            artists_name = data['artist']
        img = data.get('picUrl', None)
        return cls(pid, name, artists_name, img=img)

    def get_detail(self):
        data = self._api.album_infos(self.bid)
        if data is not None:
            data = data['album']
            self._songs = NSongModel.batch_create(data['songs'])
            self._img = data['picUrl']
            self._desc = data['briefDesc']

    @classmethod
    def get(cls, bid):
        data = cls._api.album_infos(bid)
        return cls.create(data)

    @classmethod
    def create(cls, data):
        if data is None or data['code'] != 200:
            return None
        album_data = data['album']
        bid = album_data['id']
        name = album_data['name']
        artists_name = album_data['artist']['name']
        songs = NSongModel.batch_create(album_data['songs'])
        img = album_data['picUrl']
        desc = album_data['briefDesc']
        return NAlbumModel(bid, name, artists_name, songs, img, desc)


class NArtistModel(object):
    _api = api

    def __init__(self, aid, name, img='', songs=[]):
        self.aid = aid
        self._name = name

        self._img = img
        self._songs = songs

    @property
    def name(self):
        return self._name

    @property
    def img(self):
        if not self._img:
            logger.debug('artist has no img, so get detail')
            self.get_detail()
        return self._img

    @property
    def songs(self):
        if not self._songs:
            logger.debug('artist has no songs, so get detail')
            self.get_detail()
        return self._songs

    def get_detail(self):
        data = self._api.artist_infos(self.aid)
        if data is not None:
            self._img = data['artist']['picUrl']
            self._songs = data['hotSongs']

    @classmethod
    def get(cls, aid):
        data = cls._api.artist_infos(aid)
        return cls.create(data)

    @classmethod
    def create(cls, data):
        if data is None or data['code'] != 200:
            return None

        aid = data['artist']['id']
        name = data['artist']['name']
        img = data['artist']['picUrl']

        songs = NSongModel.batch_create(data['hotSongs'])
        return cls(aid, name, img, songs)


class NUserModel(object):
    _api = api
    current_user = None

    def __init__(self, username, uid, name, img, playlists=[]):
        super().__init__()
        self.username = username

        self.uid = uid
        self.name = name
        self.img = img
        self._playlists = playlists

    def is_playlist_mine(self, pid):
        for p in self.playlists:
            if p.pid == pid:
                return True
        return False

    @classmethod
    def set_current_user(cls, user):
        cls.current_user = user

    @property
    def playlists(self):
        if self._playlists:
            return self._playlists
        data = self._api.user_playlist(self.uid)
        if data is None:
            return []
        playlists = data['playlist']
        playlists_model = []
        for p in playlists:
            model = NPlaylistModel(p['id'], p['name'], p['specialType'],
                                   p['userId'])
            playlists_model.append(model)
        self._playlists = playlists_model
        return playlists_model

    @classmethod
    def create(cls, data):
        user_data = data['data']
        username = data['username']
        img = user_data['profile']['avatarUrl']
        uid = user_data['profile']['userId']
        name = user_data['profile']['nickname']
        model = NUserModel(username, uid, name, img)
        return model

    @classmethod
    def check(cls, username, pw):
        data = cls._api.login(username, pw)
        if data is None:
            return {'code': 408, 'message': '网络状况不好'}
        elif data['code'] == 200:
            return {'code': 200, 'message': '登陆成功',
                    'data': data, 'username': username}
        elif data['code'] == 415:
            captcha_id = data['captchaId']
            url = cls._api.get_captcha_url(captcha_id)
            return {'code': 415, 'message': '本次登陆需要验证码',
                    'captcha_url': url, 'captchar_id': captcha_id}
        elif data['code'] == 501:
            return {'code': 501, 'message': '用户名不存在'}
        elif data['code'] == 502:
            return {'code': 502, 'message': '密码错误'}
        elif data['code'] == 509:
            return {'code': 509, 'message': '请休息几分钟再尝试'}

    @classmethod
    def check_captcha(cls, captcha_id, text):
        flag, cid = cls._api.confirm_captcha(captcha_id, text)
        if flag is not True:
            url = cls._api.get_captcha_url(cid)
            return {'code': 415, 'message': '验证码错误',
                    'captcha_url': url, 'captcha_id': cid}
        return {'code': 200, 'message': '验证码正确'}

    def save(self):
        with open(USERS_INFO_FILE, 'w+') as f:
            data = {
                self.username: {
                    'uid': self.uid,
                    'name': self.name,
                    'img': self.img,
                    'cookies': self._api.cookies
                }
            }
            if f.read() != '':
                data.update(json.load(f))
            json.dump(data, f, indent=4)

    @classmethod
    def load(cls):
        if not os.path.exists(USERS_INFO_FILE):
            return None
        with open(USERS_INFO_FILE, 'r') as f:
            text = f.read()
            if text == '':
                return None
            data = json.loads(text)
            username = next(iter(data.keys()))
            user_data = data[username]
            model = cls(username,
                        user_data['uid'],
                        user_data['name'],
                        user_data['img'])
            model._api.load_cookies(user_data['cookies'])
        return model

    @classmethod
    def get_recommend_songs(cls):
        if cls.current_user is None:
            return []
        data = cls._api.get_recommend_songs()
        if data.get('code') == 200:
            return NSongModel.batch_create(data['recommend'])
        return []

    @classmethod
    def get_fm_song(cls):
        if cls.current_user is None:
            return []
        data = cls._api.get_radio_music()
        if data is None:
            return []
        if data['code'] == 200:
            songs = data['data']
            return NSongModel.batch_create(songs)
        else:
            return []


class NPlaylistModel(PlaylistModel):
    instances = []
    _api = api

    def __init__(self, pid, name, ptype, uid, songs=[]):
        super().__init__()
        self.pid = pid
        self._name = name
        self.ptype = ptype
        self.uid = uid
        self._songs = songs

        NPlaylistModel.instances.append(self)

    @property
    def name(self):
        return self._name

    @property
    def songs(self):
        if self._songs:
            return self._songs
        data = self._api.playlist_detail(self.pid)
        if data is None:
            return None
        songs_model = NSongModel.batch_create(data['result']['tracks'])
        self._songs = songs_model
        return self._songs

    def update_songs(self):
        self._songs = []

    def add_song(self, mid):
        data = self._api.op_music_to_playlist(mid, self.pid, op='add')
        if data is None:
            return False
        if data['code'] == 502:
            return False
        elif data['code'] == 200:
            self.update_songs()
            return True

    def del_song(self, mid):
        data = self._api.op_music_to_playlist(mid, self.pid, op='del')
        if data.get('code') == 200:
            self.update_songs()
            return True
        return False

    @classmethod
    def del_song_from_playlist(cls, mid, pid):
        for playlist in cls.instances:
            if playlist.pid == pid:
                return playlist.del_song(mid)
        data = cls._api.op_music_to_playlist(mid, pid, op='del')
        if data['code'] == 200:
            return True
        return False



# -*- coding: utf-8 -*-

import os
import logging
import configparser

from PyQt5.QtWidgets import QWidget
from PyQt5.QtGui import QColor

from .consts import THEMES_DIR, USER_THEMES_DIR


logger = logging.getLogger(__name__)


class ThemeManager(object):
    def __init__(self, app):
        super().__init__()
        self._app = app

        self.current_theme = None
        self._themes = []    # config file name (theme name)

    def choose(self, theme_name):
        '''
        :param theme: theme unique name
        '''
        def recursive_update(widget):
            if hasattr(widget, 'set_theme_style'):
                widget.set_theme_style()
            for child in widget.children():
                if isinstance(child, QWidget):
                    recursive_update(child)

        self.set_theme(theme_name)
        recursive_update(self._app)

    def scan(self, themes_dir=[THEMES_DIR, USER_THEMES_DIR]):
        '''themes directory'''
        self._themes = []
        for directory in themes_dir:
            files = os.listdir(directory)
            for f in files:
                f_name, f_ext = f.split('.')
                if f_ext == 'colorscheme':
                    self._themes.append(f_name)

    def list(self):
        '''show themes list

        :param theme: theme unique name
        :return: themes name list
        '''
        if not self._themes:
            self.scan()
        return self._themes

    def get_theme(self, theme_name):
        '''
        :param theme: unique theme name
        :return: `Theme` object
        '''
        pass

    def set_theme(self, theme_name):
        '''set current theme'''
        self.current_theme = Theme(theme_name)


class Theme(object):
    def __init__(self, config_name=None):
        self._config = configparser.ConfigParser()
        self.name = config_name

        self.read(config_name)

    def read(self, config_file):
        config_file_path = os.path.abspath(THEMES_DIR + config_file +
                                           '.colorscheme')
        if config_file is not None:
            config = self._config.read(config_file_path)
            if config:
                return True
        return False

    @property
    def background_light(self):
        color_section = self._config['Background']
        return self._parse_color_str(color_section['color'])

    @property
    def background(self):
        color_section = self._config['BackgroundIntense']
        return self._parse_color_str(color_section['color'])

    @property
    def foreground_light(self):
        color_section = self._config['Foreground']
        return self._parse_color_str(color_section['color'])

    @property
    def foreground(self):
        color_section = self._config['ForegroundIntense']
        return self._parse_color_str(color_section['color'])

    @property
    def color0_light(self):
        color_section = self._config['Color0']
        return self._parse_color_str(color_section['color'])

    @property
    def color0(self):
        color_section = self._config['Color0Intense']
        return self._parse_color_str(color_section['color'])

    @property
    def color1_light(self):
        color_section = self._config['Color1']
        return self._parse_color_str(color_section['color'])

    @property
    def color1(self):
        color_section = self._config['Color1Intense']
        return self._parse_color_str(color_section['color'])

    @property
    def color2_light(self):
        color_section = self._config['Color2']
        return self._parse_color_str(color_section['color'])

    @property
    def color2(self):
        color_section = self._config['Color2Intense']
        return self._parse_color_str(color_section['color'])

    @property
    def color3_light(self):
        color_section = self._config['Color3']
        return self._parse_color_str(color_section['color'])

    @property
    def color3(self):
        color_section = self._config['Color3Intense']
        return self._parse_color_str(color_section['color'])

    @property
    def color4_light(self):
        color_section = self._config['Color4']
        return self._parse_color_str(color_section['color'])

    @property
    def color4(self):
        color_section = self._config['Color4Intense']
        return self._parse_color_str(color_section['color'])

    @property
    def color5_light(self):
        color_section = self._config['Color5']
        return self._parse_color_str(color_section['color'])

    @property
    def color5(self):
        color_section = self._config['Color5Intense']
        return self._parse_color_str(color_section['color'])

    @property
    def color6_light(self):
        color_section = self._config['Color6']
        return self._parse_color_str(color_section['color'])

    @property
    def color6(self):
        color_section = self._config['Color6Intense']
        return self._parse_color_str(color_section['color'])

    @property
    def color7_light(self):
        color_section = self._config['Color7']
        return self._parse_color_str(color_section['color'])

    @property
    def color7(self):
        color_section = self._config['Color7Intense']
        return self._parse_color_str(color_section['color'])

    def _parse_color_str(self, color_str):
        rgb = [int(x) for x in color_str.split(',')]
        return QColor(rgb[0], rgb[1], rgb[2])



import asyncio
import logging

from PyQt5.QtGui import QFontMetrics
from PyQt5.QtCore import Qt, QTime, pyqtSignal, pyqtSlot
from PyQt5.QtWidgets import QVBoxLayout, QHBoxLayout, QMenu, QAction
from PyQt5.QtMultimedia import QMediaPlayer

from feeluown.libs.widgets.base import FFrame, FButton, FLabel, FScrollArea,\
    FComboBox
from feeluown.libs.widgets.labels import _BasicLabel
from feeluown.libs.widgets.sliders import _BasicSlider
from feeluown.libs.widgets.components import LP_GroupHeader, LP_GroupItem, \
    MusicTable

from .utils import parse_ms
from .consts import PlaybackMode


logger = logging.getLogger(__name__)


class PlayerControlButton(FButton):
    def __init__(self, app, text=None, parent=None):
        super().__init__(text, parent)
        self._app = app

        self.setObjectName('mc_btn')
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
                font-size: 13px;
                color: {1};
                outline: none;
            }}
            #{0}:hover {{
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.foreground.name(),
                   theme.color4.name())
        self.setStyleSheet(style_str)


class ProgressSlider(_BasicSlider):
    def __init__(self, app, parent=None):
        super().__init__(app, parent)

        self.setOrientation(Qt.Horizontal)
        self.setMinimumWidth(400)
        self.setObjectName('player_progress_slider')

        self.sliderMoved.connect(self.seek)

    def set_duration(self, ms):
        self.setRange(0, ms / 1000)

    def update_state(self, ms):
        self.setValue(ms / 1000)

    def seek(self, second):
        self._app.player.setPosition(second * 1000)


class VolumeSlider(_BasicSlider):
    def __init__(self, app, parent=None):
        super().__init__(app, parent)

        self.setOrientation(Qt.Horizontal)
        self.setMinimumWidth(100)
        self.setObjectName('player_volume_slider')
        self.setRange(0, 100)   # player volume range
        self.setValue(100)
        self.setToolTip('调教播放器音量')


class ProgressLabel(_BasicLabel):
    def __init__(self, app, text=None, parent=None):
        super().__init__(app, text, parent)
        self._app = app

        self.duration_text = '00:00'

        self.setObjectName('player_progress_label')
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
            }}
        '''.format(self.objectName(),
                   theme.color3.name())
        self.setStyleSheet(self._style_str + style_str)

    def set_duration(self, ms):
        m, s = parse_ms(ms)
        duration = QTime(0, m, s)
        self.duration_text = duration.toString('mm:ss')

    def update_state(self, ms):
        m, s = parse_ms(ms)
        position = QTime(0, m, s)
        position_text = position.toString('mm:ss')
        self.setText(position_text + '/' + self.duration_text)


class PlayerControlPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self._layout = QHBoxLayout(self)
        self.previous_btn = PlayerControlButton(self._app, '上一首', self)
        self.pp_btn = PlayerControlButton(self._app, '播放', self)
        self.next_btn = PlayerControlButton(self._app, '下一首', self)
        self.progress_slider = ProgressSlider(self._app, self)
        self.volume_slider = VolumeSlider(self._app, self)
        self.progress_label = ProgressLabel(self._app, '00:00/00:00', self)

        self._btn_container = FFrame(self)
        self._slider_container = FFrame(self)

        self._bc_layout = QHBoxLayout(self._btn_container)
        self._sc_layout = QHBoxLayout(self._slider_container)

        self.setObjectName('pc_panel')
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
                color: {1};
            }}
        '''.format(self.objectName(),
                   theme.foreground.name(),
                   theme.color0.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._btn_container.setFixedWidth(140)
        self._slider_container.setMinimumWidth(700)
        self.progress_label.setFixedWidth(80)

        self._layout.setSpacing(0)
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._bc_layout.setSpacing(0)
        self._bc_layout.setContentsMargins(0, 0, 0, 0)

        self._bc_layout.addWidget(self.previous_btn)
        self._bc_layout.addStretch(1)
        self._bc_layout.addWidget(self.pp_btn)
        self._bc_layout.addStretch(1)
        self._bc_layout.addWidget(self.next_btn)

        self._sc_layout.addWidget(self.progress_slider)
        self._sc_layout.addSpacing(2)
        self._sc_layout.addWidget(self.progress_label)
        self._sc_layout.addSpacing(5)
        self._sc_layout.addStretch(0)
        self._sc_layout.addWidget(self.volume_slider)
        self._sc_layout.addStretch(1)

        self._layout.addWidget(self._btn_container)
        self._layout.addSpacing(10)
        self._layout.addWidget(self._slider_container)


class SongOperationPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.setObjectName('song_operation_panel')
        self.set_theme_style()

    def set_theme_style(self):
        style_str = '''
            #{0} {{
                background: transparent;
            }}
        '''.format(self.objectName())
        self.setStyleSheet(style_str)


class TopPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self._layout = QHBoxLayout(self)
        self.pc_panel = PlayerControlPanel(self._app, self)
        self.mo_panel = SongOperationPanel(self._app, self)

        self.setObjectName('top_panel')
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
                color: {1};
                border-bottom: 3px inset {3};
            }}
        '''.format(self.objectName(),
                   theme.foreground.name(),
                   theme.color0_light.name(),
                   theme.color0_light.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)
        self.setFixedHeight(50)
        self._layout.addSpacing(5)
        self._layout.addWidget(self.pc_panel)
        self._layout.addWidget(self.mo_panel)


class LP_LibraryPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.header = LP_GroupHeader(self._app, '我的音乐')
        self.current_playlist_item = LP_GroupItem(self._app, '当前播放列表')
        self.current_playlist_item.set_img_text('❂')
        self._layout = QVBoxLayout(self)

        self.setObjectName('lp_library_panel')
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
            }}
        '''.format(self.objectName(),
                   theme.color3.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        self._layout.addSpacing(3)
        self._layout.addWidget(self.header)
        self._layout.addWidget(self.current_playlist_item)

    def add_item(self, item):
        self._layout.addWidget(item)


class LP_PlaylistsPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.header = LP_GroupHeader(self._app, '歌单')
        self._layout = QVBoxLayout(self)
        self.setObjectName('lp_playlists_panel')

        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
            }}
        '''.format(self.objectName(),
                   theme.color5.name())
        self.setStyleSheet(style_str)

    def add_item(self, item):
        self._layout.addWidget(item)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        self._layout.addWidget(self.header)


class LeftPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.library_panel = LP_LibraryPanel(self._app)
        self.playlists_panel = LP_PlaylistsPanel(self._app)

        self._layout = QVBoxLayout(self)
        self.setLayout(self._layout)
        self.setObjectName('c_left_panel')
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
            }}
        '''.format(self.objectName(),
                   theme.color5.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        self._layout.addWidget(self.library_panel)
        self._layout.addWidget(self.playlists_panel)
        self._layout.addStretch(1)


class LeftPanel_Container(FScrollArea):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.left_panel = LeftPanel(self._app)
        self._layout = QVBoxLayout(self)  # no layout, no children
        self.setWidget(self.left_panel)

        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setWidgetResizable(True)

        self.setObjectName('c_left_panel_container')
        self.set_theme_style()
        self.setMinimumWidth(180)
        self.setMaximumWidth(220)

        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
                border: 0px;
                border-right: 3px inset {1};
            }}
        '''.format(self.objectName(),
                   theme.color0_light.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        # self._layout.addWidget(self.left_panel)


class RightPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.widget = None

        self._layout = QHBoxLayout(self)
        self.setLayout(self._layout)
        self.setObjectName('right_panel')
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        style_str = '''
            #{0} {{
                background: transparent;
            }}
        '''.format(self.objectName())
        self.setStyleSheet(style_str)

    def set_widget(self, widget):
        if self.widget and self.widget != widget:
            self._layout.removeWidget(self.widget)
            self.widget.hide()
            widget.show()
            self._layout.addWidget(widget)
        else:
            self._layout.addWidget(widget)
        self.widget = widget

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)


class RightPanel_Container(FScrollArea):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.right_panel = RightPanel(self._app)
        self._layout = QVBoxLayout(self)
        self.setWidget(self.right_panel)

        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setWidgetResizable(True)

        self.setObjectName('c_left_panel')
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
                border: 0px;
            }}
        '''.format(self.objectName(),
                   theme.color5.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)


class CentralPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.left_panel_container = LeftPanel_Container(self._app, self)
        self.right_panel_container = RightPanel_Container(self._app, self)
        self.left_panel = self.left_panel_container.left_panel
        self.right_panel = self.right_panel_container.right_panel

        self._layout = QHBoxLayout(self)
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        style_str = '''
            #{0} {{
                background: transparent;
            }}
        '''.format(self.objectName())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        self._layout.addWidget(self.left_panel_container)
        self._layout.addWidget(self.right_panel_container)


class SongLabel(FLabel):
    def __init__(self, app, text=None, parent=None):
        super().__init__(text, parent)
        self._app = app

        self.setObjectName('song_label')
        self.setIndent(5)
        self.set_theme_style()

        self.set_song('No song is playing')

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.color7.name(),
                   theme.color0.name())
        self.setStyleSheet(style_str)

    def set_song(self, song_text):
        self.setText('♪  ' + song_text + ' ')


class PlaybackModeSwitchBtn(FButton):
    def __init__(self, app, parent=None):
        super().__init__(parent=parent)
        self._app = app

        self.setObjectName('player_mode_switch_btn')
        self.set_theme_style()
        self.set_text('循环')

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
                border: 0px;
                padding: 0px 4px;
            }}
        '''.format(self.objectName(),
                   theme.color6.name(),
                   theme.background.name())
        self.setStyleSheet(style_str)

    def set_text(self, text):
        self.setText('♭ ' + text)

    def on_playback_mode_changed(self, playback_mode):
        if playback_mode == PlaybackMode.sequential:
            self.set_text(self._app.player_mode_manager.current_mode.name)
        else:
            self.set_text(playback_mode.value)


class ThemeComboBox(FComboBox):
    clicked = pyqtSignal()
    signal_change_theme = pyqtSignal([str])

    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.setObjectName('theme_switch_btn')
        self.setEditable(False)
        self.maximum_width = 150
        self.set_theme_style()
        self.setFrame(False)
        self.current_theme = self._app.theme_manager.current_theme.name
        self.themes = [self.current_theme]
        self.set_themes(self.themes)

        self.currentIndexChanged.connect(self.on_index_changed)

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
                border: 0px;
                padding: 0px 4px;
                border-radius: 0px;
            }}
            #{0}::drop-down {{
                width: 0px;
                border: 0px;
            }}
            #{0} QAbstractItemView {{
                border: 0px;
                min-width: 200px;
            }}
        '''.format(self.objectName(),
                   theme.color4.name(),
                   theme.background.name(),
                   theme.foreground.name())
        self.setStyleSheet(style_str)

    @pyqtSlot(int)
    def on_index_changed(self, index):
        if index < 0 or not self.themes:
            return
        metrics = QFontMetrics(self.font())
        if self.themes[index] == self.current_theme:
            return
        self.current_theme = self.themes[index]
        name = '❀ ' + self.themes[index]
        width = metrics.width(name)
        if width < self.maximum_width:
            self.setFixedWidth(width + 10)
            self.setItemText(index, name)
            self.setToolTip(name)
        else:
            self.setFixedWidth(self.maximum_width)
            text = metrics.elidedText(name, Qt.ElideRight,
                                      self.width())
            self.setItemText(index, text)
            self.setToolTip(text)
        self.signal_change_theme.emit(self.current_theme)

    def add_item(self, text):
        self.addItem('❀ ' + text)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton and \
                self.rect().contains(event.pos()):
            self.clicked.emit()
            self.showPopup()

    def set_themes(self, themes):
        self.clear()
        if self.current_theme:
            self.themes = [self.current_theme]
            self.add_item(self.current_theme)
        else:
            self.themes = []
        for theme in themes:
            if theme not in self.themes:
                self.add_item(theme)
                self.themes.append(theme)


class PlayerStateLabel(FLabel):
    def __init__(self, app, text=None, parent=None):
        super().__init__('♫', parent)
        self._app = app

        self.setObjectName('player_state_label')
        self.setToolTip('这里显示的是播放器的状态\n'
                        'Buffered 代表该音乐已经可以开始播放\n'
                        'Loading 代表正在加载该音乐\n'
                        'Failed 代表加载音乐失败')
        self.set_theme_style()

    def set_text(self, text):
        self.setText(('♫ ' + text).upper())

    @property
    def common_style(self):
        style_str = '''
            #{0} {{
                padding-left: 3px;
                padding-right: 5px;
            }}
        '''.format(self.objectName())
        return style_str

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.color6_light.name(),
                   theme.color7.name())
        self.setStyleSheet(style_str + self.common_style)

    def set_error_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.color1_light.name(),
                   theme.color7.name())
        self.setStyleSheet(style_str + self.common_style)

    def set_normal_style(self):
        self.set_theme_style()

    def update_media_state(self, state):
        self.set_theme_style()
        logger.debug('current player media state %d' % state)
        if state == QMediaPlayer.LoadedMedia:
            self.set_text('Loaded')
        elif state == QMediaPlayer.LoadingMedia:
            self.set_text('Loading')
        elif state == QMediaPlayer.BufferingMedia:
            self.set_text('Buffering')
        elif state == QMediaPlayer.BufferedMedia:
            self.set_text('Buffered')
        elif state == QMediaPlayer.InvalidMedia:
            self.set_text('Failed')

    def update_state(self, state):
        return
        self.set_theme_style()
        if state == QMediaPlayer.StoppedState:
            self.set_text('Stopped')
        elif state == QMediaPlayer.PlayingState:
            self.set_text('Playing')
        elif state == QMediaPlayer.PausedState:
            self.set_text('Paused')

    def set_error(self, error):
        self.set_error_style()
        if error == QMediaPlayer.ResourceError:
            self.set_text('Decode Failed')
        elif error == QMediaPlayer.NetworkError:
            self.set_text('Network Error')
        elif error == QMediaPlayer.FormatError:
            self.set_text('Decode Failed')
        elif error == QMediaPlayer.ServiceMissingError:
            self.set_text('Gsteamer Missing')
        else:
            logger.error('player error %d' % error)


class MessageLabel(FLabel):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.setObjectName('message_label')
        self.queue = []
        self.hide()

    @property
    def common_style(self):
        style_str = '''
            #{0} {{
                padding-left: 3px;
                padding-right: 5px;
            }}
        '''.format(self.objectName())
        return style_str

    def _set_error_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.color1_light.name(),
                   theme.color7_light.name())
        self.setStyleSheet(style_str + self.common_style)

    def _set_normal_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.color6_light.name(),
                   theme.foreground.name())
        self.setStyleSheet(style_str + self.common_style)

    def show_message(self, text, error=False):
        if self.isVisible():
            self.queue.append({'error': error, 'message': text})
            return
        if error:
            self._set_error_style()
            self.setText(text)
        else:
            self._set_normal_style()
            self.setText(text)
        self.show()
        app_event_loop = asyncio.get_event_loop()
        app_event_loop.call_later(3, self.access_message_queue)

    def access_message_queue(self):
        self.hide()
        if self.queue:
            m = self.queue.pop(0)
            self.show_message(m['message'], m['error'])


class AppStatusLabel(FLabel):
    clicked = pyqtSignal()

    def __init__(self, app, text=None, parent=None):
        super().__init__(text, parent)
        self._app = app

        self.setText('♨ Normal'.upper())
        self.setToolTip('点击可以切换到其他模式哦 ~\n'
                        '不过暂时还没实现这个功能...敬请期待 ~')
        self.setObjectName('app_status_label')
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {3};
                padding-left: 5px;
                padding-right: 5px;
                font-size: 14px;
            }}
            #{0}:hover {{
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.color4.name(),
                   theme.color2.name(),
                   theme.background.name())
        self.setStyleSheet(style_str)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton and \
                self.rect().contains(event.pos()):
            self.clicked.emit()


class NetworkStatus(FLabel):
    def __init__(self, app, text=None, parent=None):
        super().__init__(text, parent)
        self._app = app

        self.setToolTip('这里显示的是当前网络状态')
        self.setObjectName('network_status_label')
        self.set_theme_style()

        self.set_state(1)

    @property
    def common_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
                padding-left: 5px;
                padding-right: 5px;
                font-size: 16px;
                font-weight: bold;
            }}
        '''.format(self.objectName(),
                   theme.color3.name(),
                   theme.background.name())
        return style_str

    def set_theme_style(self):
        self.setStyleSheet(self.common_style)

    def _set_error_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
            }}
        '''.format(self.objectName(),
                   theme.color5.name())
        self.setStyleSheet(self.common_style + style_str)

    def _set_normal_style(self):
        self.setStyleSheet(self.common_style)

    def set_state(self, state):
        if state == 0:
            self._set_error_style()
            self.setText('✕')
        elif state == 1:
            self._set_normal_style()
            self.setText('✓')


class StatusPanel(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self._layout = QHBoxLayout(self)
        self.player_state_label = PlayerStateLabel(self._app)
        self.app_status_label = AppStatusLabel(self._app)
        self.network_status_label = NetworkStatus(self._app)
        self.message_label = MessageLabel(self._app)
        self.song_label = SongLabel(self._app, parent=self)
        self.pms_btn = PlaybackModeSwitchBtn(self._app, self)
        self.theme_switch_btn = ThemeComboBox(self._app, self)

        self.setup_ui()
        self.setObjectName('status_panel')
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
            }}
        '''.format(self.objectName(),
                   theme.color0.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)
        self.setFixedHeight(18)
        # self.song_label.setMinimumWidth(220)
        self.song_label.setMaximumWidth(300)
        self._layout.addWidget(self.player_state_label)
        self._layout.addWidget(self.app_status_label)
        self._layout.addWidget(self.network_status_label)
        self._layout.addStretch(0)
        self._layout.addWidget(self.message_label)
        self._layout.addStretch(0)
        self._layout.addWidget(self.theme_switch_btn)
        self._layout.addWidget(self.pms_btn)
        self._layout.addWidget(self.song_label)


class CurrentPlaylistTable(MusicTable):
    remove_signal = pyqtSignal([int])   # song id

    def __init__(self, app):
        super().__init__(app)
        self._app = app

        self._row = 0

        self.menu = QMenu()
        self.remove = QAction('从当前列表中移除', self)
        self.menu.addAction(self.remove)

        self.remove.triggered.connect(self.remove_song)

    def contextMenuEvent(self, event):
        point = event.pos()
        item = self.itemAt(point)
        if item is not None:
            row = self.row(item)
            self._row = row
            self.menu.exec(event.globalPos())

    def remove_song(self):
        song = self.songs[self._row]
        self.songs.pop(self._row)
        self.removeRow(self._row)
        self.remove_signal.emit(song.mid)


class LyricFrame(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app


class Ui(object):
    def __init__(self, app):
        self._layout = QVBoxLayout(app)
        self.top_panel = TopPanel(app, app)
        self.central_panel = CentralPanel(app, app)
        self.status_panel = StatusPanel(app, app)
        self.current_playlist_table = CurrentPlaylistTable(app)

        self.setup()

    def setup(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)
        self._layout.addWidget(self.top_panel)
        self._layout.addWidget(self.central_panel)
        self._layout.addWidget(self.status_panel)



import json
import logging
import os

from PyQt5.QtCore import QObject
from PyQt5.QtGui import QKeySequence

from .api import api
from .consts import USER_PW_FILE
from .fm_player_mode import FM_mode
from .model import NUserModel, NSongModel
from .ui import Ui, SongsTable, PlaylistItem

logger = logging.getLogger(__name__)


class Nem(QObject):

    def __init__(self, app):
        super().__init__(parent=app)
        self._app = app
        api.set_http(self._app.request)

        self.ui = Ui(self._app)

        self.user = None

        self.registe_hotkey()
        self.init_signal_binding()

    def init_signal_binding(self):
        self.ui.login_btn.clicked.connect(self.ready_to_login)
        self.ui.login_dialog.ok_btn.clicked.connect(self.login)
        self.ui.songs_table_container.table_control.play_all_btn.clicked\
            .connect(self.play_all)
        self.ui.songs_table_container.table_control.search_box.textChanged\
            .connect(self.search_table)
        self.ui.songs_table_container.table_control.search_box.returnPressed\
            .connect(self.search_net)

        self.ui.fm_item.clicked.connect(self.enter_fm_mode)
        self.ui.recommend_item.clicked.connect(self.show_recommend_songs)

    def enter_fm_mode(self):
        mode = FM_mode(self._app)
        self._app.player_mode_manager.enter_mode(mode)

    def registe_hotkey(self):
        self._app.hotkey_manager.registe(
            QKeySequence('Ctrl+F'),
            self.ui.songs_table_container.table_control.search_box.setFocus)

    def show_recommend_songs(self):
        songs = NUserModel.get_recommend_songs()
        songs_table = SongsTable(self._app)
        self.load_songs(songs, songs_table)

    def load_user_pw(self):
        if not os.path.exists(USER_PW_FILE):
            return
        with open(USER_PW_FILE, 'r') as f:
            d = json.load(f)
            data = d[d['default']]
        self.ui.login_dialog.username_input.setText(data['username'])
        self.ui.login_dialog.pw_input.setText(data['password'])
        self.ui.login_dialog.is_encrypted = True

        logger.info('load username and password from %s' % USER_PW_FILE)

    def save_user_pw(self, data):
        with open(USER_PW_FILE, 'w+') as f:
            if f.read() == '':
                d = dict()
            else:
                d = json.load(f)
            d['default'] = data['username']
            d[d['default']] = data
            json.dump(d, f, indent=4)

        logger.info('save username and password to %s' % USER_PW_FILE)

    def ready_to_login(self):
        model = NUserModel.load()
        if model is None:
            self.ui.login_dialog.show()
            self.load_user_pw()
        else:
            logger.info('load last user.')
            self.user = model
            NUserModel.set_current_user(model)
            self._on_login_in()

    def login(self):
        login_dialog = self.ui.login_dialog
        if login_dialog.captcha_needed:
            captcha = str(login_dialog.captcha_input.text())
            captcha_id = login_dialog.captcha_id
            data = NUserModel.check_captcha(captcha_id, captcha)
            if data['code'] == 200:
                login_dialog.captcha_input.hide()
                login_dialog.captcha_label.hide()
            else:
                login_dialog.captcha_verify(data)

        user_data = login_dialog.data
        self.ui.login_dialog.show_hint('正在登录...')
        data = NUserModel.check(user_data['username'], user_data['password'])
        message = data['message']
        self.ui.login_dialog.show_hint(message)
        if data['code'] == 200:
            # login in
            self.save_user_pw(user_data)
            self.user = NUserModel.create(data)
            self._on_login_in()
        elif data['code'] == 415:
            login_dialog.captcha_verify(data)

    def _on_login_in(self):
        logger.info('login in... set user infos.')

        self.ui.on_login_in()

        self.ui.login_btn.set_avatar(self.user.img)
        self.user.save()
        self.load_playlists()

    def load_playlists(self):
        playlist_widget = self._app.ui.central_panel.left_panel.playlists_panel
        for playlist in self.user.playlists:
            item = PlaylistItem(self._app, playlist)
            item.load_playlist_signal.connect(self.load_playlist)
            playlist_widget.add_item(item)

    def play_song(self, song):
        self._app.player.play(song)

    def play_all(self):
        songs_table = self.ui.songs_table_container.songs_table
        if songs_table is not None:
            self._app.player.set_music_list(songs_table.songs)

    def play_mv(self, mvid):
        pass

    def search_table(self, text):
        songs_table = self.ui.songs_table_container.songs_table
        songs_table.search(text)

    def search_net(self):
        text = self.ui.songs_table_container.table_control.search_box.text()
        songs = NSongModel.search(text)
        self._app.message('搜索到 %d 首相关歌曲' % len(songs))
        if songs:
            self.load_songs(songs)

    def load_songs(self, songs, songs_table=None):
        if songs_table is None:
            songs_table = SongsTable(self._app)
        songs_table.set_songs(songs)
        songs_table.play_song_signal.connect(self.play_song)
        songs_table.play_mv_signal.connect(self.play_mv)
        songs_table.show_artist_signal.connect(self.load_artist)
        songs_table.show_album_signal.connect(self.load_album)
        songs_table.add_song_signal.connect(self._app.player.add_music)
        songs_table.set_to_next_signal.connect(
            self._app.player.set_tmp_fixed_next_song)
        self.ui.songs_table_container.set_table(songs_table)
        self._app.ui.central_panel.right_panel.set_widget(
            self.ui.songs_table_container)

    def load_playlist(self, playlist):
        logger.info('load playlist : %d, %s' % (playlist.pid, playlist.name))
        if playlist.songs is None:
            return
        songs_table = SongsTable(self._app)
        songs_table.set_playlist_id(playlist.pid)
        self.load_songs(playlist.songs, songs_table)

    def load_artist(self, aid):
        print('aid:', aid)
        pass

    def load_album(self, bid):
        print('bid:', bid)
        pass



# -*- coding: utf-8 -*-

from .base import FSlider
from feeluown.utils import lighter


class _BasicSlider(FSlider):
    '''slider with basic style'''
    def __init__(self, app, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._app = app
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        self._style_str = '''
            QSlider::groove:horizontal {{
                background: {1};
                height: 4px;
                border-radius: 2px;
            }}
            QSlider::handle:horizontal {{
                background: {2};
                width: 16px;
                margin: -6px 0;
                border-radius: 8px;
            }}
            QSlider::sub-page:horizontal {{
                background: {3};
            }}
        '''.format(self.objectName(),
                   lighter(theme.color0_light).name(),
                   theme.color4.name(),
                   theme.foreground.name())
        self.setStyleSheet(self._style_str)



#! /usr/bin/env python3

import asyncio
import os
import sys

sys.path.append(os.path.dirname(sys.path[0]))

from PyQt5.QtWidgets import QApplication
from quamash import QEventLoop

from feeluown.app import App
from feeluown.consts import (HOME_DIR, USER_PLUGINS_DIR, PLUGINS_DIR, DATA_DIR,
                             CACHE_DIR, USER_THEMES_DIR)
from feeluown.config import config
from feeluown import logger_config


def parse_args(args):
    if '-d' in args:
        config.debug = True
    logger_config()


def ensure_dir():
    if not os.path.exists(HOME_DIR):
        os.mkdir(HOME_DIR)
    if not os.path.exists(DATA_DIR):
        os.mkdir(DATA_DIR)
    if not os.path.exists(USER_PLUGINS_DIR):
        os.mkdir(USER_PLUGINS_DIR)
    if not os.path.exists(CACHE_DIR):
        os.mkdir(CACHE_DIR)
    if not os.path.exists(USER_THEMES_DIR):
        os.mkdir(USER_THEMES_DIR)


ensure_dir()
os.chdir(os.path.join(os.path.dirname(os.path.abspath(__file__)), '..'))
sys.path.append(PLUGINS_DIR)
sys.path.append(USER_PLUGINS_DIR)


def main():
    parse_args(sys.argv)

    q_app = QApplication(sys.argv)
    q_app.setQuitOnLastWindowClosed(True)
    q_app.setApplicationName('FeelUOwn')

    app_event_loop = QEventLoop(q_app)
    asyncio.set_event_loop(app_event_loop)
    app = App()
    app.show()

    app_event_loop.run_forever()
    sys.exit(0)


if __name__ == '__main__':
    main()



class SongModel(object):
    def __init__(self):
        pass

    @property
    def mid(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def title(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def artists_name(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def album_name(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def album_img(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def url(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def length(self):
        raise NotImplementedError('This should be implemented by subclass')


class MvModel(object):
    def __init__(self):
        pass

    @property
    def song_model(self):
        return None

    @property
    def url(self):
        raise NotImplementedError('This should be implemented by subclass')


class ArtistModel(object):
    def __init__(self):
        pass

    @property
    def name(self):
        raise NotImplementedError('This should be implemented by subclass')


class AlbumModel(object):
    def __init__(self):
        pass

    @property
    def name(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def artists_name(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def img(self):
        return ''

    @property
    def songs(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def desc(self):
        return ''


class PlaylistModel(object):
    def __init__(self):
        pass

    @property
    def name(self):
        raise NotImplementedError('This should be implemented by subclass')

    @property
    def songs(self):
        raise NotImplementedError('This should be implemented by subclass')



# -*- coding: utf-8 -*-

from .base import FLabel


class _BasicLabel(FLabel):
    '''label with basic style'''
    def __init__(self, app, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._app = app

        theme = app.theme_manager.current_theme
        self._style_str = '''
            QLabel {{
                background: transparent;
                color: {0};
            }}
        '''.format(theme.foreground.name())
        self.set_theme_style()

    def set_theme_style(self):
        self.setStyleSheet(self._style_str)



import logging
import platform
import time
from functools import wraps
from PyQt5.QtGui import QColor


logger = logging.getLogger(__name__)


def parse_ms(ms):
    minute = int(ms / 60000)
    second = int((ms % 60000) / 1000)
    return minute, second


def lighter(color, degree=1, a=255):
    r, g, b = color.red(), color.green(), color.blue()
    r = r + 10 * degree if (r + 10 * degree) < 255 else 255
    g = g + 10 * degree if (g + 10 * degree) < 255 else 255
    b = b + 10 * degree if (b + 10 * degree) < 255 else 255
    return QColor(r, g, b, a)


def darker(color, degree=1, a=255):
    r, g, b = color.red(), color.green(), color.blue()
    r = r - 10 * degree if (r - 10 * degree) > 0 else 0
    g = g - 10 * degree if (g - 10 * degree) > 0 else 0
    b = b - 10 * degree if (b - 10 * degree) > 0 else 0
    return QColor(r, g, b, a)


def measure_time(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        t = time.process_time()
        result = func(*args, **kwargs)
        elapsed_time = time.process_time() - t
        logger.info('function %s executed time: %f s'
                    % (func.__name__, elapsed_time))
        return result
    return wrapper


def is_linux():
    if platform.system() == 'Linux':
        return True
    return False


def is_osx():
    if platform.system() == 'Darwin':
        return True
    return False



# -*- coding: utf-8 -*-

import logging

from .nem import Nem

__alias__ = '网易云音乐'
__version__ = '0.0.1'
__desc__ = '网易云音乐'


def enable(app):
    logger = logging.getLogger(__name__)
    Nem(app)
    logger.info('neteasemusic plugin enabled')


def disable(app):
    logger = logging.getLogger(__name__)
    logger.info('neteasemusic plugin disabled')



# -*- coding: utf-8 -*-

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QHBoxLayout, QAbstractItemView, QHeaderView, \
    QTableWidgetItem

from .base import FFrame, FLabel, FTableWidget
from feeluown.model import SongModel
from feeluown.utils import darker, parse_ms, measure_time


class LP_GroupHeader(FFrame):
    def __init__(self, app, title=None, parent=None):
        super().__init__(parent)
        self._app = app

        self._layout = QHBoxLayout(self)
        self.title_label = FLabel(title, self)
        self.title_label.setIndent(8)

        self.setObjectName('lp_group_header')
        self.set_theme_style()
        self.setup_ui()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
            }}

            #{0} QLabel {{
                font-size: 12px;
                color: {1};
            }}
        '''.format(self.objectName(),
                   theme.foreground_light.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)
        self.setFixedHeight(22)

        self._layout.addWidget(self.title_label)

    def set_header(self, text):
        self.title_label.setText(text)


class LP_GroupItem(FFrame):
    clicked = pyqtSignal()

    def __init__(self, app, name=None, parent=None):
        super().__init__(parent)
        self._app = app

        self.is_selected = False
        self.is_playing = False

        self._layout = QHBoxLayout(self)
        self._flag_label = FLabel(self)
        self._img_label = FLabel(self)
        self._name_label = FLabel(name, self)

        self.setObjectName('lp_group_item')
        self._flag_label.setObjectName('lp_groun_item_flag')
        self._flag_label.setText('➣')
        self._flag_label.setIndent(5)
        self._img_label.setObjectName('lp_group_item_img')
        self._img_label.setText('♬')
        self._name_label.setObjectName('lp_group_item_name')
        self.set_theme_style()
        self.setup_ui()

    def set_img_text(self, text):
        self._img_label.setText(text)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton and \
                self.rect().contains(event.pos()):
            self.clicked.emit()

    def enterEvent(self, event):
        theme = self._app.theme_manager.current_theme
        label_hover_color = theme.color4
        if self.is_selected or self.is_playing:
            return
        self._img_label.setStyleSheet(
            'color: {0};'.format(label_hover_color.name()))
        self._name_label.setStyleSheet(
            'color: {0};'.format(label_hover_color.name()))

    def leaveEvent(self, event):
        theme = self._app.theme_manager.current_theme
        label_color = theme.foreground
        if self.is_selected or self.is_playing:
            return
        self._img_label.setStyleSheet('color: {0};'.format(label_color.name()))
        self._name_label.setStyleSheet('color: {0};'.format(label_color.name()))

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
            }}
            #{1} {{
                color: transparent;
                font-size: 14px;
            }}
            #{2} {{
                color: {4};
                font-size: 14px;
            }}
            #{3} {{
                color: {4};
                font-size: 13px;
            }}
        '''.format(self.objectName(),
                   self._flag_label.objectName(),
                   self._img_label.objectName(),
                   self._name_label.objectName(),
                   theme.foreground.name(),
                   theme.color0.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)
        self.setFixedHeight(26)
        self._img_label.setFixedWidth(18)
        self._flag_label.setFixedWidth(22)

        self._layout.addWidget(self._flag_label)
        self._layout.addWidget(self._img_label)
        self._layout.addSpacing(2)
        self._layout.addWidget(self._name_label)

    def set_selected(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
            }}
            #{1} {{
                color: {4};
                font-size: 14px;
            }}
            #{2} {{
                color: {5};
                font-size: 14px;
            }}
            #{3} {{
                color: {6};
                font-size: 13px;
            }}
        '''.format(self.objectName(),
                   self._flag_label.objectName(),
                   self._img_label.objectName(),
                   self._name_label.objectName(),
                   theme.color5_light.name(),
                   theme.color6.name(),
                   theme.color3_light.name())
        self.setStyleSheet(style_str)


class MusicTable(FTableWidget):
    play_song_signal = pyqtSignal([SongModel])

    def __init__(self, app, rows=0, columns=6, parent=None):
        super().__init__(rows, columns, parent)
        self._app = app

        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self._alignment = Qt.AlignLeft | Qt.AlignVCenter
        self.horizontalHeader().setDefaultAlignment(self._alignment)
        self.verticalHeader().hide()
        self.setShowGrid(False)

        self.setObjectName('music_table')
        self.set_theme_style()
        self.songs = []

        self.setHorizontalHeaderLabels(['', '歌曲名', '歌手', '专辑', '时长',
                                        ''])
        self.setColumnWidth(0, 28)
        self.setColumnWidth(2, 150)
        self.setColumnWidth(3, 200)
        self.setColumnWidth(5, 100)
        self.horizontalHeader().setSectionResizeMode(1, QHeaderView.Stretch)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.cellDoubleClicked.connect(self.on_cell_dbclick)

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            QHeaderView {{
                color: {1};
                background: transparent;
                font-size: 14px;
            }}
            QHeaderView::section:horizontal {{
                height: 24px;
                background: transparent;
                border-top: 1px;
                border-right: 1px;
                border-bottom: 1px;
                border-color: {5};
                color: {5};
                border-style: solid;
                padding-left: 5px;
            }}

            QTableView QTableCornerButton::section {{
                background: transparent;
                border: 0px;
                border-bottom: 1px solid {1};
            }}
            #{0} {{
                border: 0px;
                background: transparent;
                color: {1};
            }}
            #{0}::item {{
                outline: none;
            }}
            #{0}::item:focus {{
                background: transparent;
                outline: none;
            }}
            #{0}::item:selected {{
                background: {4};
            }}
        '''.format(self.objectName(),
                   theme.foreground.name(),
                   theme.color6.name(),
                   darker(theme.background, a=50).name(QColor.HexArgb),
                   theme.color0.name(),
                   theme.color7_light.name())
        self.setStyleSheet(style_str)

    def add_item(self, song_model):
        music_item = QTableWidgetItem(song_model.title)
        album_item = QTableWidgetItem(song_model.album_name)
        artist_item = QTableWidgetItem(song_model.artists_name)
        m, s = parse_ms(song_model.length)
        length_item = QTableWidgetItem(str(m) + ':' + str(s))

        row = self.rowCount()
        self.setRowCount(row + 1)
        self.setItem(row, 1, music_item)
        self.setItem(row, 2, artist_item)
        self.setItem(row, 3, album_item)
        self.setItem(row, 4, length_item)

        self.songs.append(song_model)

    def set_songs(self, songs):
        self.setRowCount(0)
        for song in songs:
            self.add_item(song)

    @measure_time
    def search(self, text):
        if not text:
            for i in range(self.rowCount()):
                self.showRow(i)
            return
        for i, song in enumerate(self.songs):
            if text.lower() not in song.title.lower()\
                    and text not in song.album_name.lower()\
                    and text not in song.artists_name.lower():
                self.hideRow(i)
            else:
                self.showRow(i)

    def on_cell_dbclick(self, row, column):
        song = self.songs[row]
        if column == 0:
            pass
        elif column == 1:
            self.play_song_signal.emit(song)
        elif column == 2:
            pass
        elif column == 3:
            pass

    def keyPressEvent(self, event):
        self.setFocus()     # gain focus from cell widget if neccesary
        key_code = event.key()
        if key_code == Qt.Key_J:
            self.setCurrentCell(self._next_row(), 1)
        elif key_code == Qt.Key_K:
            self.setCurrentCell(self._prev_row(), 1)
        elif key_code in (Qt.Key_Enter, Qt.Key_Return):
            current_row = self.currentRow()
            self.play_song_signal.emit(self.songs[current_row])
        else:
            super().keyPressEvent(event)

    def _next_row(self):
        current_row = self.currentRow()
        return current_row + 1 if current_row != (self.rowCount() - 1) else 0

    def _prev_row(self):
        current_row = self.currentRow()
        return current_row - 1 if current_row != 0 else self.rowCount() - 1



# -*- coding: utf-8 -*-

from PyQt5.QtWidgets import QWidget, QFrame, QPushButton, QLabel, QSlider,\
    QScrollArea, QDialog, QLineEdit, QCheckBox, QTableWidget, QComboBox
from PyQt5.QtCore import QObject


class FButton(QPushButton):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def set_theme_style(self):
        pass


class FCheckBox(QCheckBox):
    def __init__(self, parent=None):
        super().__init__(parent)

    def set_theme_style(self):
        pass


class FComboBox(QComboBox):
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)

    def set_theme_style(self):
        pass


class FDialog(QDialog):
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)

    def set_theme_style(self):
        pass


class FFrame(QFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def set_theme_style(self):
        self.setStyleSheet('background: transparent;')


class FLabel(QLabel):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def set_theme_style(self):
        pass


class FLineEdit(QLineEdit):
    def __init__(self, parent=None):
        super().__init__(parent)

    def set_theme_style(self):
        pass


class FObject(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)

    def set_theme_style(self):
        pass


class FSlider(QSlider):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def set_theme_style(self):
        pass


class FScrollArea(QScrollArea):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def set_theme_style(self):
        pass


class FTableWidget(QTableWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def set_theme_style(self):
        pass


class FWidget(QWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def set_theme_style(self):
        pass



import asyncio
import logging
from functools import partial


logger = logging.getLogger(__name__)


class VersionManager(object):
    current_version = 'v9.1a'

    def __init__(self, app):
        self._app = app

    @asyncio.coroutine
    def check_release(self):
        url = 'https://api.github.com/repos/cosven/FeelUOwn/releases'
        logger.info('正在查找新版本...')
        try:
            loop = asyncio.get_event_loop()
            future = loop.run_in_executor(
                None, partial(self._app.request.get, url, timeout=5))
            res = yield from future
            if not res.status_code == 200:
                logger.warning('connect to api.github.com timeout')
                return
            releases = res.json()
            for release in releases:
                if release['tag_name'] > self.current_version:
                    title = u'发现新版本 %s hoho' % release['tag_name']
                    logger.info(title)
                    self._app.message(title)
                    break
        except Exception as e:
            logger.error(str(e))



# -*- coding: utf-8 -*-

import asyncio
import logging
import random

from PyQt5.QtMultimedia import QMediaPlayer, QMediaContent
from PyQt5.QtCore import QUrl, pyqtSignal, pyqtSlot

from .model import SongModel
from .consts import PlaybackMode


logger = logging.getLogger(__name__)


class Player(QMediaPlayer):
    signal_player_media_changed = pyqtSignal([SongModel])
    signal_playlist_is_empty = pyqtSignal()
    signal_playback_mode_changed = pyqtSignal([PlaybackMode])
    signal_playlist_finished = pyqtSignal()

    signal_song_required = pyqtSignal()
    finished = pyqtSignal()

    _music_list = list()    # 里面的对象是music_model
    _current_index = None
    _tmp_fix_next_song = None
    playback_mode = PlaybackMode.loop
    last_playback_mode = PlaybackMode.loop
    _other_mode = False

    def __init__(self, app):
        super().__init__(app)
        self._app = app

        self.error.connect(self.on_error_occured)
        self.mediaChanged.connect(self.on_media_changed)
        self.mediaStatusChanged.connect(self.on_media_status_changed)

        self._music_error_times = 0

        # latency of retying next operation when error happened
        self._retry_latency = 3
        # when _music_error_times reached _music_error_maximum, play next music
        self._music_error_maximum = 3

    def change_player_mode_to_normal(self):
        logger.debug('退出特殊的播放模式')
        self._other_mode = False
        self._set_playback_mode(self.last_playback_mode)

    def change_player_mode_to_other(self):
        # player mode: such as fm mode, different from playback mode
        logger.debug('进入特殊的播放模式')
        self._other_mode = True
        self._set_playback_mode(PlaybackMode.sequential)

    def _record_playback_mode(self):
        self.last_playback_mode = self.playback_mode

    @pyqtSlot(QMediaContent)
    def on_media_changed(self, media_content):
        music_model = self._music_list[self._current_index]
        self.signal_player_media_changed.emit(music_model)

    @pyqtSlot(QMediaPlayer.MediaStatus)
    def on_media_status_changed(self, state):
        if state == QMediaPlayer.EndOfMedia:
            self.finished.emit()
            self.stop()
            if (self._current_index == len(self._music_list) - 1) and\
                    self._other_mode:
                self.signal_playlist_finished.emit()
                logger.debug("播放列表播放完毕")
            if not self._other_mode:
                self.play_next()
        # TODO: at hotkey linux module, when it call
        #       Controller.player.play_next or last may stop the player
        #       add following code to fix the problem.
        elif state in (QMediaPlayer.LoadedMedia, ):
            self.play()

    def insert_to_next(self, model):
        if not self.is_music_in_list(model):
            if self._current_index is None:
                index = 0
            else:
                index = self._current_index + 1
            self._music_list.insert(index, model)
            return True
        return False

    def add_music(self, song):
        self._music_list.append(song)

    def remove_music(self, mid):
        for i, music_model in enumerate(self._music_list):
            if mid == music_model.mid:
                self._music_list.pop(i)
                if i == self._current_index:
                    self.play_next()
                elif i < self._current_index:
                    self._current_index -= 1
                return True
        return False

    def get_media_content_from_model(self, music_model):
        url = music_model.url
        if not url:
            self._app.message('URL 不存在，不能播放该歌曲')
            return None
        if url.startswith('http'):
            media_content = QMediaContent(QUrl(url))
        else:
            media_content = QMediaContent(QUrl.fromLocalFile(url))
        return media_content

    def set_music_list(self, music_list):
        self._music_list = []
        self._music_list = music_list
        if len(self._music_list):
            self.play(self._music_list[0])

    def clear_playlist(self):
        self._music_list = []
        self._current_index = 0
        self.stop()

    def is_music_in_list(self, model):
        for music in self._music_list:
            if model.mid == music.mid:
                return True
        return False

    def _play(self, music_model):
        insert_flag = self.insert_to_next(music_model)
        index = self.get_index_by_model(music_model)
        if not insert_flag and self._current_index is not None:
            if music_model.mid == self._music_list[self._current_index].mid\
                    and self.state() == QMediaPlayer.PlayingState:
                return True
        super().stop()
        media_content = self.get_media_content_from_model(music_model)
        if media_content is not None:
            logger.debug('start to play song: %d, %s, %s' %
                         (music_model.mid, music_model.title, music_model.url))
            self._current_index = index
            self.setMedia(media_content)
            super().play()
            return True
        else:
            self.remove_music(music_model.mid)
            self.play_next()
            return False

    def other_mode_play(self, music_model):
        self._play(music_model)

    def play(self, music_model=None):
        if music_model is None:
            super().play()
            return False
        self._app.player_mode_manager.exit_to_normal()
        self._play(music_model)

    def get_index_by_model(self, music_model):
        for i, music in enumerate(self._music_list):
            if music_model.mid == music.mid:
                return i
        return None

    def play_or_pause(self):
        if len(self._music_list) is 0:
            self.signal_playlist_is_empty.emit()
            return
        if self.state() == QMediaPlayer.PlayingState:
            self.pause()
        elif self.state() == QMediaPlayer.PausedState:
            self.play()
        else:
            self.play_next()

    def play_next(self):
        if self._tmp_fix_next_song is not None:
            flag = self.play(self._tmp_fix_next_song)
            self._tmp_fix_next_song = None
            return flag
        index = self.get_next_song_index()
        if index is not None:
            if index == 0 and self._other_mode:
                self.signal_playlist_finished.emit()
                logger.debug("播放列表播放完毕")
                return
            music_model = self._music_list[index]
            self.play(music_model)
            return True
        else:
            self.signal_playlist_is_empty.emit()
            return False

    def play_last(self):
        index = self.get_previous_song_index()
        if index is not None:
            music_model = self._music_list[index]
            self.play(music_model)
            return True
        else:
            self.signal_playlist_is_empty.emit()
            return False

    def set_tmp_fixed_next_song(self, song):
        self._tmp_fix_next_song = song

    @pyqtSlot(QMediaPlayer.Error)
    def on_error_occured(self, error):
        song = self._music_list[self._current_index]
        self._app.message('cant play song: %s' % (song.title))
        self.stop()
        if error == QMediaPlayer.FormatError:
            self._app.message('这首歌挂了，也有可能是断网了', error=True)
            logger.debug('song cant be played, url is %s' % song.url)
        elif error == QMediaPlayer.NetworkError:
            self._wait_to_retry()
        elif error == QMediaPlayer.ResourceError:
            logger.error('网络出现错误：不能正确解析资源')
        elif error == QMediaPlayer.ServiceMissingError:
            self._app.notify('缺少解码器，请向作者求助', error=True)
        else:
            self._wait_to_next(2)

    def _wait_to_retry(self):
        if self._music_error_times >= self._music_error_maximum:
            self._music_error_times = 0
            self._wait_to_next(self._retry_latency)
            self._app.message('将要播放下一首歌曲', error=True)
        else:
            self._music_error_times += 1
            app_event_loop = asyncio.get_event_loop()
            app_event_loop.call_later(self._retry_latency, self.play)
            self._app.message('网络连接不佳', error=True)

    def _wait_to_next(self, second=0):
        if len(self._music_list) < 2:
            return
        app_event_loop = asyncio.get_event_loop()
        app_event_loop.call_later(second, self.play_next)

    def get_next_song_index(self):
        if len(self._music_list) is 0:
            self._app.message('当前播放列表没有歌曲')
            return None
        if self.playback_mode == PlaybackMode.one_loop:
            return self._current_index
        elif self.playback_mode == PlaybackMode.loop:
            if self._current_index >= len(self._music_list) - 1:
                return 0
            else:
                return self._current_index + 1
        elif self.playback_mode == PlaybackMode.sequential:
            return None
        else:
            return random.choice(range(len(self._music_list)))

    def get_previous_song_index(self):
        if len(self._music_list) is 0:
            return None
        if self.playback_mode == PlaybackMode.one_loop:
            return self._current_index
        elif self.playback_mode == PlaybackMode.loop:
            if self._current_index is 0:
                return len(self._music_list) - 1
            else:
                return self._current_index - 1
        elif self.playback_mode == PlaybackMode.sequential:
            return None
        else:
            return random.choice(range(len(self._music_list)))

    def _set_playback_mode(self, mode):
        # item once: 0
        # item in loop: 1
        # sequential: 2
        # loop: 3
        # random: 4
        if mode == self.playback_mode:
            return 0
        self._record_playback_mode()
        self.playback_mode = mode
        self._app.message('设置播放顺序为：%s' % mode.value)
        self.signal_playback_mode_changed.emit(mode)

    def next_playback_mode(self):
        if self.playback_mode == PlaybackMode.one_loop:
            self._set_playback_mode(PlaybackMode.loop)
        elif self.playback_mode == PlaybackMode.loop:
            self._set_playback_mode(PlaybackMode.random)
        elif self.playback_mode == PlaybackMode.random:
            self._set_playback_mode(PlaybackMode.one_loop)

    @property
    def songs(self):
        return self._music_list



#! /usr/bin/env python3

import platform
import shutil
import os


def install_sys_dep():

    def _get_gstreamer(gstreamer_version='0.10'):
        v_package = ['gstreamer{0}-plugins-good'.format(gstreamer_version),
                     'gstreamer{0}-plugins-bad'.format(gstreamer_version),
                     'gstreamer{0}-plugins-ugly'.format(gstreamer_version)]
        return v_package

    packages = ['python3-pyqt5',
                'python3-pyqt5.qtmultimedia',
                'libqt5multimedia5-plugins',
                'fcitx-frontend-qt5']

    linux_distro = platform.linux_distribution()
    if linux_distro[0] in ('Deepin') or 'ubuntu' in linux_distro[0].lower():
        print('Download and install software dependency.')
        os.system('sudo apt-get install -y {packages} 1>/dev/null'
                  .format(packages=' '.join(packages)))
        if linux_distro[1] >= '16.04':
            gstreamer_version = '1.0'
        else:
            gstreamer_version = '0.10'
        gstreamer_packages = _get_gstreamer(gstreamer_version)
        packages = packages + gstreamer_packages
        os.system('sudo apt-get install -y {packages} 1>/dev/null'
                  .format(packages=' '.join(packages)))
        print('\ninstall finished ~')
    else:
        print('You should install these packages by yourself, '
              'as your linux may not a debian distribution\n')
        packages = packages + _get_gstreamer()
        for i, p in enumerate(packages):
            print(i, ': ', p)
        print('\npackage name may be different among different systems')


def generate_icon():
    print('Generate icon, then you can see app in apps list.')
    DESKTOP_FILE = 'feeluown.desktop'
    current_path = os.getcwd()
    feeluown_dir = current_path + '/feeluown'
    feeluown_icon = feeluown_dir + '/feeluown.png'
    icon_string = '#!/usr/bin/env xdg-open\n'\
                  '[Desktop Entry]\n'\
                  'Type=Application\n'\
                  'Name=FeelUOwn\n'\
                  'Comment=FeelUOwn Launcher\n'\
                  'Exec=sh -c "python3 -m feeluown"\n'\
                  'Icon={feeluown_icon}\n'\
                  'Categories=AudioVideo;Audio;Player;Qt;\n'\
                  'Terminal=false\n'\
                  'StartupNotify=true\n'\
                  .format(feeluown_icon=feeluown_icon,
                          current_path=current_path)
    f_path = os.path.expanduser('~') +\
        '/.local/share/applications/' + DESKTOP_FILE
    try:
        with open(f_path, 'w') as f:
            f.write(icon_string)
        os.system('chmod +x %s' % f_path)
    except PermissionError:
        print('Please use sudo')
    en_desktop_path = os.path.expanduser('~') + '/Desktop'
    cn_desktop_path = os.path.expanduser('~') + '/桌面'
    if os.path.exists(en_desktop_path):
        desktop_f = en_desktop_path + '/' + DESKTOP_FILE
    if os.path.exists(cn_desktop_path):
        desktop_f = cn_desktop_path + '/' + DESKTOP_FILE
    shutil.copy(f_path, desktop_f)
    os.system('chmod +x %s' % desktop_f)
    print('gen finished')


def update():
    os.system('sudo -H pip3 install --upgrade feeluown')
    os.system('feeluown-install-dev')
    os.system('feeluown-genicon')

if __name__ == '__main__':
    install_sys_dep()
    generate_icon()



import hashlib
import logging

from PyQt5.QtCore import pyqtSignal, Qt
from PyQt5.QtWidgets import (QHBoxLayout, QVBoxLayout, QLineEdit, QHeaderView,
                             QMenu, QAction, QAbstractItemView)

from feeluown.libs.widgets.base import FLabel, FFrame, FDialog, FLineEdit, \
    FButton
from feeluown.libs.widgets.components import MusicTable, LP_GroupItem

from .model import NPlaylistModel, NSongModel, NUserModel


logger = logging.getLogger(__name__)


class LineInput(FLineEdit):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.setObjectName('line_input')
        self.set_theme_style()

    def set_theme_style(self):
        pass


class LoginDialog(FDialog):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.is_encrypted = False
        self.captcha_needed = False
        self.captcha_id = 0

        self.username_input = LineInput(self)
        self.pw_input = LineInput(self)
        self.pw_input.setEchoMode(QLineEdit.Password)
        # self.remember_checkbox = FCheckBox(self)
        self.captcha_label = FLabel(self)
        self.captcha_label.hide()
        self.captcha_input = LineInput(self)
        self.captcha_input.hide()
        self.hint_label = FLabel(self)
        self.ok_btn = FButton('登录', self)
        self._layout = QVBoxLayout(self)

        self.username_input.setPlaceholderText('网易邮箱或者手机号')
        self.pw_input.setPlaceholderText('密码')

        self.setObjectName('login_dialog')
        self.set_theme_style()
        self.setup_ui()

        self.pw_input.textChanged.connect(self.dis_encrypt)

    def fill(self, data):
        self.username_input.setText(data['username'])
        self.pw_input.setText(data['password'])
        self.is_encrypted = True

    def set_theme_style(self):
        pass

    def setup_ui(self):
        self.setFixedWidth(200)

        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        self._layout.addWidget(self.username_input)
        self._layout.addWidget(self.pw_input)
        self._layout.addWidget(self.captcha_label)
        self._layout.addWidget(self.captcha_input)
        self._layout.addWidget(self.hint_label)
        # self._layout.addWidget(self.remember_checkbox)
        self._layout.addWidget(self.ok_btn)

    def show_hint(self, text):
        self.hint_label.setText(text)

    @property
    def data(self):
        username = self.username_input.text()
        pw = self.pw_input.text()
        if self.is_encrypted:
            password = pw
        else:
            password = hashlib.md5(pw.encode('utf-8')).hexdigest()
        d = dict(username=username, password=password)
        return d

    def captcha_verify(self, data):
        self.captcha_needed = True
        url = data['captcha_url']
        self.captcha_id = data['captcha_id']
        self.captcha_input.show()
        self.captcha_label.show()
        self._app.pixmap_from_url(url, self.captcha_label.setPixmap)

    def dis_encrypt(self, text):
        self.is_encrypted = False


class LoginButton(FLabel):
    clicked = pyqtSignal()

    def __init__(self, app, text=None, parent=None):
        super().__init__(text, parent)
        self._app = app

        self.setText('登录')
        self.setToolTip('登陆网易云音乐')
        self.setObjectName('nem_login_btn')
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: transparent;
                color: {1};
            }}
            #{0}:hover {{
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.foreground.name(),
                   theme.color4.name())
        self.setStyleSheet(style_str)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton and \
                self.rect().contains(event.pos()):
            self.clicked.emit()

    def set_avatar(self, url):
        pixmap = self._app.pixmap_from_url(url)
        if pixmap is not None:
            self.setPixmap(pixmap.scaled(self.size(),
                           transformMode=Qt.SmoothTransformation))


class PlaylistItem(LP_GroupItem):
    load_playlist_signal = pyqtSignal(NPlaylistModel)

    def __init__(self, app, playlist=None, parent=None):
        super().__init__(app, playlist.name, parent=parent)
        self._app = app

        self.model = playlist
        self.clicked.connect(self.on_clicked)
        self.setAcceptDrops(True)

    def on_clicked(self):
        self.load_playlist_signal.emit(self.model)

    def dropEvent(self, event):
        source = event.source()
        if not isinstance(source, SongsTable):
            return
        event.accept()
        song = source.drag_song
        if song is not None:
            user = NUserModel.current_user
            if user.is_playlist_mine(self.model.pid):
                self.add_song_to_playlist(song)

    def add_song_to_playlist(self, song):
        logger.debug('temp to add "%s" to playlist "%s"' %
                     (song.title, self.model.name))
        if self.model.add_song(song.mid):
            self._app.message('add "%s" to playlist "%s" success' %
                              (song.title, self.model.name))
        else:
            self._app.message('add "%s" to playlist "%s" failed' %
                              (song.title, self.model.name), error=True)

    def dragEnterEvent(self, event):
        event.accept()

    def dragMoveEvent(self, event):
        event.accept()


class SongsTable(MusicTable):
    play_mv_signal = pyqtSignal([int])
    play_song_signal = pyqtSignal([NSongModel])
    show_artist_signal = pyqtSignal([int])
    show_album_signal = pyqtSignal([int])
    add_song_signal = pyqtSignal([NSongModel])
    set_to_next_signal = pyqtSignal([NSongModel])

    def __init__(self, app, rows=0, columns=6, parent=None):
        super().__init__(app, rows, columns, parent)
        self._app = app

        self.setObjectName('nem_songs_table')
        self.set_theme_style()
        self.songs = []

        self.setHorizontalHeaderLabels(['', '歌曲名', '歌手', '专辑', '时长',
                                        ''])
        self.setColumnWidth(0, 28)
        self.setColumnWidth(2, 150)
        self.setColumnWidth(3, 200)
        self.setColumnWidth(5, 100)
        self.horizontalHeader().setSectionResizeMode(1, QHeaderView.Stretch)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        self.cellDoubleClicked.connect(self.on_cell_dbclick)
        self.setDragEnabled(True)
        self.setDragDropMode(QAbstractItemView.DragOnly)

        self._context_menu_row = 0
        self._drag_row = None
        self._playlist_id = 0

    def add_song_to_current_playlist(self):
        song = self.songs[self._context_menu_row]
        self.add_song_signal.emit(song)

    def set_song_to_next(self):
        song = self.songs[self._context_menu_row]
        self.set_to_next_signal.emit(song)

    def remove_song_from_playlist(self):
        song = self.songs[self._context_menu_row]
        if NPlaylistModel.del_song_from_playlist(song.mid, self._playlist_id):
            self.removeRow(self._context_menu_row)
            self._app.message('删除 %s 成功' % song.title)
        else:
            self._app.message('删除 %s 失败' % song.title, error=True)

    def set_playlist_id(self, pid):
        self._playlist_id = pid

    def is_playlist(self):
        if self._playlist_id:
            return True
        return False

    def _is_playlist_mine(self):
        if self.is_playlist():
            user = NUserModel.current_user
            if user.is_playlist_mine(self._playlist_id):
                return True
        return False

    def contextMenuEvent(self, event):
        menu = QMenu()
        add_to_current_playlist_action = QAction('添加到当前播放列表', self)
        set_song_next_to_action = QAction('下一首播放', self)
        menu.addAction(add_to_current_playlist_action)
        menu.addAction(set_song_next_to_action)

        if self._is_playlist_mine():
            remove_song_from_playlist_action = QAction('从歌单中删除该歌曲', self)
            menu.addAction(remove_song_from_playlist_action)
            remove_song_from_playlist_action.triggered.connect(
                self.remove_song_from_playlist)

        add_to_current_playlist_action.triggered.connect(
            self.add_song_to_current_playlist)
        set_song_next_to_action.triggered.connect(
            self.set_song_to_next)

        point = event.pos()
        item = self.itemAt(point)
        if item is not None:
            row = self.row(item)
            self._context_menu_row = row
            menu.exec(event.globalPos())

    @property
    def drag_song(self):
        if self._drag_row is not None:
            return self.songs[self._drag_row]
        return None

    def mousePressEvent(self, event):
        super().mousePressEvent(event)
        point = event.pos()
        item = self.itemAt(point)
        if item is not None:
            self._drag_row = self.row(item)

    def on_cell_dbclick(self, row, column):
        song = self.songs[row]
        if column == 0:
            if NSongModel.mv_available(song.mvid):
                self.play_mv_signal.emit(song.mvid)
        elif column == 1:
            self.play_song_signal.emit(song)
        elif column == 2:
            self.show_artist_signal.emit(song.artists[0].aid)
        elif column == 3:
            self.show_album_signal.emit(song.album.bid)


class SearchBox(FLineEdit):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.setObjectName('search_box')
        self.setPlaceholderText('搜索歌曲、歌手')
        self.setToolTip('输入文字可以从当前歌单内搜索\n'
                        '按下 Enter 将搜索网络')
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            #{0} {{
                padding-left: 3px;
                font-size: 14px;
                background: transparent;
                border: 0px;
                border-bottom: 1px solid {1};
                color: {2};
                outline: none;
            }}
            #{0}:focus {{
                outline: none;
            }}
        '''.format(self.objectName(),
                   theme.color6.name(),
                   theme.foreground.name())
        self.setStyleSheet(style_str)


class TableControl(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.play_all_btn = FButton('▶')
        self.search_box = SearchBox(self._app)
        self._layout = QHBoxLayout(self)
        self.setup_ui()
        self.setObjectName('n_table_control')
        self.set_theme_style()

    def set_theme_style(self):
        theme = self._app.theme_manager.current_theme
        style_str = '''
            QPushButton {{
                background: transparent;
                color: {1};
                font-size: 16px;
                outline: none;
            }}
            QPushButton:hover {{
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.foreground.name(),
                   theme.color0.name())
        self.setStyleSheet(style_str)

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)
        self.setFixedHeight(40)
        self.play_all_btn.setFixedSize(20, 20)
        self.search_box.setFixedSize(160, 26)

        self._layout.addSpacing(20)
        self._layout.addWidget(self.play_all_btn)
        self._layout.addStretch(0)
        self._layout.addWidget(self.search_box)
        self._layout.addSpacing(60)


class SongsTable_Container(FFrame):
    def __init__(self, app, parent=None):
        super().__init__(parent)
        self._app = app

        self.songs_table = None
        self.table_control = TableControl(self._app)
        self._layout = QVBoxLayout(self)
        self.setup_ui()

    def setup_ui(self):
        self._layout.setContentsMargins(0, 0, 0, 0)
        self._layout.setSpacing(0)

        self._layout.addWidget(self.table_control)

    def set_table(self, songs_table):
        if self.songs_table:
            self._layout.replaceWidget(self.songs_table, songs_table)
            self.songs_table.deleteLater()
        else:
            self._layout.addWidget(songs_table)
            self._layout.addSpacing(10)
        self.songs_table = songs_table


class Ui(object):
    def __init__(self, app):
        super().__init__()
        self._app = app

        self.login_dialog = LoginDialog(self._app, self._app)
        self.login_btn = LoginButton(self._app)
        self._lb_container = FFrame()
        self.songs_table_container = SongsTable_Container(self._app)
        self.fm_item = LP_GroupItem(self._app, '私人FM')
        self.fm_item.set_img_text('Ω')
        self.recommend_item = LP_GroupItem(self._app, '每日推荐')
        self.recommend_item.set_img_text('✦')

        self._lbc_layout = QHBoxLayout(self._lb_container)

        self.setup()

    def setup(self):

        self._lbc_layout.setContentsMargins(0, 0, 0, 0)
        self._lbc_layout.setSpacing(0)

        self._lbc_layout.addWidget(self.login_btn)
        self.login_btn.setFixedSize(30, 30)
        self._lbc_layout.addSpacing(10)

        tp_layout = self._app.ui.top_panel.layout()
        tp_layout.addWidget(self._lb_container)

    def on_login_in(self):
        self.login_dialog.close()

        library_panel = self._app.ui.central_panel.left_panel.library_panel
        library_panel.add_item(self.fm_item)
        library_panel.add_item(self.recommend_item)



from PyQt5.QtWidgets import QShortcut


class Hotkey(object):
    def __init__(self, app):
        self._app = app

    def registe(self, key_sequence, callback):
        shortcut = QShortcut(key_sequence, self._app)
        shortcut.activated.connect(callback)



#!/usr/bin/env python3

from setuptools import setup


setup(
    name='feeluown',
    version='1.0.2',
    description='*nix music player',
    author='Cosven',
    author_email='cosven.yin@gmail.com',
    packages=[
        'feeluown',
        'feeluown.libs',
        'feeluown.libs.widgets',
        'feeluown.plugins.neteasemusic',
        ],
    package_data={
        '': ['themes/*.colorscheme', '*.png']
        },
    url='https://github.com/cosven/FeelUOwn',
    keywords=['media', 'player', 'application', 'PyQt5', 'python3'],
    classifiers=(
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3 :: Only',
        'Environment :: X11 Applications :: Qt',
        ),
    # FIXME depends on PyQt5 , but cannot put that in a setup.py
    install_requires=[
        'quamash>=0.5.5',
        ],
    entry_points={
        'console_scripts': [
                "feeluown=feeluown.__main__:main",
                "feeluown-install-dev=feeluown.install:install_sys_dep",
                "feeluown-genicon=feeluown.install:generate_icon",
                "feeluown-update=feeluown.install:update"
            ]
        },
    )



import asyncio

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPainter, QImage, QPixmap, QIcon
from PyQt5.QtWidgets import QApplication
from PyQt5.QtMultimedia import QMediaPlayer

from .consts import DEFAULT_THEME_NAME, APP_ICON
from .hotkey import Hotkey
from .player import Player
from .player_mode import PlayerModeManager
from .plugin import PluginsManager
from .request import Request
from .theme import ThemeManager
from .ui import Ui
from .utils import darker
from .version import VersionManager
from feeluown.libs.widgets.base import FFrame


class App(FFrame):
    def __init__(self):
        super().__init__()
        self.player = Player(self)
        self.player_mode_manager = PlayerModeManager(self)
        self.request = Request(self)
        self.theme_manager = ThemeManager(self)
        self.hotkey_manager = Hotkey(self)
        self.plugins_manager = PluginsManager(self)
        self.version_manager = VersionManager(self)
        self.theme_manager.set_theme(DEFAULT_THEME_NAME)

        self.ui = Ui(self)
        self._init_managers()

        self.player_pixmap = None

        self.resize(960, 600)
        self.setObjectName('app')
        QApplication.setWindowIcon(QIcon(APP_ICON))
        self.set_theme_style()

        self.bind_signal()
        self.test()

    def bind_signal(self):
        top_panel = self.ui.top_panel
        status_panel = self.ui.status_panel
        library_panel = self.ui.central_panel.left_panel.library_panel

        self.player.stateChanged.connect(self._on_player_status_changed)
        self.player.positionChanged.connect(self._on_player_position_changed)
        self.player.durationChanged.connect(self._on_player_duration_changed)
        self.player.signal_player_media_changed.connect(
            self._on_player_song_changed)
        self.player.mediaStatusChanged.connect(
            status_panel.player_state_label.update_media_state)
        self.player.stateChanged.connect(
            status_panel.player_state_label.update_state)
        self.player.error.connect(status_panel.player_state_label.set_error)
        self.player.signal_playback_mode_changed.connect(
            status_panel.pms_btn.on_playback_mode_changed)

        status_panel.pms_btn.clicked.connect(self.player.next_playback_mode)
        status_panel.theme_switch_btn.signal_change_theme.connect(
            self.theme_manager.choose)
        status_panel.theme_switch_btn.clicked.connect(
            self.refresh_themes)

        self.request.connected_signal.connect(self._on_network_connected)
        self.request.disconnected_signal.connect(self._on_network_disconnected)
        self.request.slow_signal.connect(self._on_network_slow)
        self.request.server_error_signal.connect(self._on_network_server_error)

        top_panel.pc_panel.volume_slider.sliderMoved.connect(
            self.change_volume)
        top_panel.pc_panel.pp_btn.clicked.connect(self.player.play_or_pause)
        top_panel.pc_panel.next_btn.clicked.connect(self.player.play_next)
        top_panel.pc_panel.previous_btn.clicked.connect(self.player.play_last)

        library_panel.current_playlist_item.clicked.connect(
            self.show_current_playlist)
        self.ui.current_playlist_table.play_song_signal.connect(
            self.player.play)
        self.ui.current_playlist_table.remove_signal.connect(
            self.player.remove_music)

    def paintEvent(self, event):
        painter = QPainter(self)
        bg_color = darker(self.theme_manager.current_theme.background, a=200)

        if self.player_pixmap is not None:
            pixmap = self.player_pixmap.scaled(
                self.size(),
                Qt.KeepAspectRatioByExpanding,
                Qt.SmoothTransformation)
            painter.drawPixmap(0, 0, pixmap)
            painter.fillRect(self.rect(), bg_color)

    def _init_managers(self):
        self.plugins_manager.scan()
        asyncio.Task(self.version_manager.check_release())

    def set_theme_style(self):
        theme = self.theme_manager.current_theme
        style_str = '''
            #{0} {{
                background: {1};
                color: {2};
            }}
        '''.format(self.objectName(),
                   theme.background.name(),
                   theme.foreground.name())
        self.setStyleSheet(style_str)

    def message(self, text, error=False):
        self.ui.status_panel.message_label.show_message(text, error)

    def notify(self, text, error=False):
        pass

    def test(self):
        # self.theme_manager.choose('Molokai')
        # self.theme_manager.choose('Tomorrow Night')
        pass

    def _on_player_duration_changed(self, ms):
        self.ui.top_panel.pc_panel.progress_label.set_duration(ms)
        self.ui.top_panel.pc_panel.progress_slider.set_duration(ms)

    def _on_player_position_changed(self, ms):
        self.ui.top_panel.pc_panel.progress_label.update_state(ms)
        self.ui.top_panel.pc_panel.progress_slider.update_state(ms)

    def _on_player_song_changed(self, song):
        song_label = self.ui.status_panel.song_label
        song_label.set_song(song.title + ' - ' + song.artists_name)
        self.player_pixmap = self.pixmap_from_url(song.album_img)
        if self.player_pixmap is not None:
            QApplication.setWindowIcon(QIcon(self.player_pixmap))
        self.update()

    def _on_player_status_changed(self, status):
        pp_btn = self.ui.top_panel.pc_panel.pp_btn
        if status == QMediaPlayer.PlayingState:
            pp_btn.setText('暂停')
        else:
            pp_btn.setText('播放')

    def _on_network_slow(self):
        network_status_label = self.ui.status_panel.network_status_label
        self.message('网络连接超时', error=True)
        network_status_label.set_state(0)

    def _on_network_connected(self):
        network_status_label = self.ui.status_panel.network_status_label
        network_status_label.set_state(1)

    def _on_network_server_error(self):
        self.message('服务端出现错误', error=True)

    def _on_network_disconnected(self):
        network_status_label = self.ui.status_panel.network_status_label
        self.message('网络连接失败', error=True)
        network_status_label.set_state(0)

    def change_volume(self, value):
        self.player.setVolume(value)

    def show_current_playlist(self):
        self.ui.current_playlist_table.set_songs(self.player.songs)
        right_panel = self.ui.central_panel.right_panel
        right_panel.set_widget(self.ui.current_playlist_table)

    def refresh_themes(self):
        theme_switch_btn = self.ui.status_panel.theme_switch_btn
        themes = self.theme_manager.list()
        theme_switch_btn.set_themes(themes)

    def pixmap_from_url(self, url, callback=None):
        res = self.request.get(url)
        if res is None:
            return None
        img = QImage()
        img.loadFromData(res.content)
        if callback is not None:
            callback(QPixmap(img))
            return None
        else:
            return QPixmap(img)

    def closeEvent(self, event):
        self.player.stop()
        QApplication.quit()



# -*- coding: utf-8 -*-

import logging

from .consts import LOG_FILE
from .config import config


__version__ = '1.0.2'


def logger_config():
    if config.debug:
        logging.basicConfig(
            level=logging.DEBUG,
            format="[%(levelname)s] [%(filename)s line:%(lineno)d] : %(message)s",
        )
    else:
        logging.basicConfig(
            format="[%(levelname)s] [%(filename)s line:%(lineno)d] : %(message)s",
            level=logging.DEBUG,
            filename=LOG_FILE,
            filemode='w',
        )
